/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die grundlegende (default) Umsetzung der AnzeigeQuerschnitt-Informationen.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.5 $ / $Date: 2014/02/13 16:27:47 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeQuerschnittTypUmsetzungTlsDefault
{  
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Status ok.
   */
  final static int STATUS_OK = 0;
  /**
   * Status TeilStrung.
   */
  final static int STATUS_TEILSTOERUNG = 1;
  /**
   * Status gestrt.
   */
  final static int STATUS_GESTOERT = 2;
  /**
   * Status Stromausfall.
   */
  final static int STATUS_STROMAUSFALL = 3;
  /**
   * Status Trkontakt.
   */
  final static int STATUS_TRKONTAKT = 4;
  /**
   * Status undefiniert.
   */
  final static int STATUS_UNDEFINIERT = 5;
  /**
   * Status Kommunikationsausfall.
   */
  final static int STATUS_KOMMUNIKATIONSAUSFALL = 6;
  /**
   * HashMap mit AnzeigeQuerschnittAktorTls mit Key DEs.
   */
  protected HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTlsDes = null; 
  /**
   * HashMap mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  protected HashMap<String, AnzeigeQuerschnittAktorTls> m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt = null;
  /**
   * HashMap mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   */
  protected HashMap<String, Data> m_listeAnzeigeQuerschnittEigenschaftIst = null;
  /**
   * HashMap mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   */
  protected HashMap<String, Data> m_listeAnzeigeQuerschnittSchaltbarkeitNba = null;
  /**
   * HashMap fr Status Methoden fr alle AnzeigenQuerschnitte.
   */
  protected HashMap<String, HashMap<String, Integer>> m_listeAnzeigeQuerschnittStatus = null;

  enum Betriebsart
  {
    Normalbetrieb,
    Blindbetrieb,
    Autarkbetrieb,
    Tunnelbetrieb,
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeQuerschnittAktorTlsDes Liste mit AnzeigeQuerschnittAktorTls mit Key DEs.
   * @param listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt Liste mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   * @param listeAnzeigeQuerschnittEigenschaftIst Liste mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittSchaltbarkeitNba Liste mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittStatus Liste mit Status Methoden fr alle AnzeigenQuerschnitte
   */
  public AnzeigeQuerschnittTypUmsetzungTlsDefault(HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsDes, HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, HashMap<String, Data> listeAnzeigeQuerschnittEigenschaftIst, HashMap<String, Data> listeAnzeigeQuerschnittSchaltbarkeitNba, HashMap<String, HashMap<String, Integer>> listeAnzeigeQuerschnittStatus)
  {
    m_listeAnzeigeQuerschnittAktorTlsDes = listeAnzeigeQuerschnittAktorTlsDes;
    m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt = listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt;
    m_listeAnzeigeQuerschnittEigenschaftIst = listeAnzeigeQuerschnittEigenschaftIst;
    m_listeAnzeigeQuerschnittSchaltbarkeitNba = listeAnzeigeQuerschnittSchaltbarkeitNba;
    m_listeAnzeigeQuerschnittStatus = listeAnzeigeQuerschnittStatus;

    m_debug.finer("AnzeigeTypUmsetzungTlsDefault");
  }
  
  /**
   * Abarbeitung des DE-Fehler genau nach TLS.
   * 
   * @param result Attributgruppe atg.tlsGloDeFehler.
   */
  public Data tlsGloDeFehler(ResultData result)
  {
    SystemObject object = result.getObject();
    
    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsGloDeFehler DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsGloDeFehler", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
  
        String deFehlerStatus = data.getTextValue("DEFehlerStatus").getValueText();
        
        if (deFehlerStatus.equals("ok"))
        {
          dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnittPid, "tlsGloDeFehler", STATUS_OK));
        }
        else
        {
          AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsGloDeFehler", STATUS_GESTOERT));
        }
  
        String deKanalStatus = data.getTextValue("DEKanalStatus").getValueText();
        String passiviert = deKanalStatus.equals("aktiv") ? "Nein" : "Ja";
        dataAnzeigeQuerschnittEigenschaftIst.getItem("Eigenschaft").getScaledValue("Passiviert").setText(passiviert);
      }
      else // keine Daten vorhanden
      {
        AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsGloDeFehler", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeQuerschnittEigenschaftIst;
  }
  
  /**
   * Abarbeitung der WZG-Helligkeit genau nach TLS.
   * 
   * @param result Attributgruppe atg.tlsWzgHelligkeit.
   */
  public Data tlsWzgHelligkeit(ResultData result)
  {
    SystemObject object = result.getObject();
    
    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsWzgHelligkeit DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgHelligkeit", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        String helligkeit = data.getScaledValue("HelligkeitsWert").getValueText();
        
        dataAnzeigeQuerschnittEigenschaftIst.getItem("Eigenschaft").getScaledValue("Helligkeit").setText(helligkeit);
        
        dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnittPid, "tlsWzgHelligkeit", STATUS_OK));
      }
      else // keine Daten vorhanden
      {
        AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgHelligkeit", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeQuerschnittEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Betriebsart genau nach TLS 93.
   * 
   * @param result Attributgruppe atg.tlsWzgBetriebsart.
   */
  public Data tlsWzgBetriebsArt(ResultData result)
  {
    SystemObject object = result.getObject();
    
    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsWzgBetriebsart DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        String betriebsart = data.getScaledValue("BetriebsArt").getValueText();
        
        String betriebszustand = null;
        
        if (betriebsart.equals("Normalbetrieb"))
        {
          betriebszustand = "AQNormalbetrieb";
        }
        
        if (betriebsart.equals("Blindbetrieb"))
        {
          betriebszustand = "AQBlindbetrieb";
        }
        
        if (betriebsart.equals("Handbetrieb"))
        {
          betriebszustand = "AQVorortbetrieb";
        }
        
        if (betriebsart.equals("Autarker Betrieb"))
        {
          betriebszustand = "AQAutarkbetrieb";
        }
        
        if (betriebsart.equals("Testbetrieb"))
        {
          betriebszustand = "AQTestbetrieb";
        }
        
        if (betriebsart.equals("Notbetrieb"))
        {
          betriebszustand = "AQNotbetrieb";
        }
        
        if (betriebszustand == null)
        {
          betriebszustand = "Undefiniert";
        }
        
        dataAnzeigeQuerschnittEigenschaftIst.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText(betriebszustand);
        
        dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_OK));
      }
      else // keine Daten vorhanden
      {
        AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeQuerschnittEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Erweiterung der Betriebsart fr die NBA Kln-Koblenz in den Unterzentralen von Heusch-Boesefeldt.
   * Hier wird die Information des vorgelagerten Entscheiders ausgewertet.
   * 
   * @param result Attributgruppe atg.tlsWzgBetriebsart mit Aspekt asp.tlsVorgabeBedienung.
   */
  public Data tlsWzgBetriebsArtNba(ResultData result)
  {
    SystemObject object = result.getObject();

    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittSchaltbarkeitNba = (Data)getListeAnzeigeQuerschnittSchaltbarkeitNba().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsWzgBetriebsart DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittSchaltbarkeitNba);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), "nicht ermittelbar"));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        String betriebsart = data.getScaledValue("BetriebsArt").getValueText();
        
        String betriebszustand = null;
        
        if (betriebsart.equals("Normalbetrieb"))
        {
          betriebszustand = "Ja";
        }
        
        if (betriebsart.equals("Handbetrieb"))
        {
          betriebszustand = "Nein";
        }
        
        if (betriebszustand == null)
        {
          betriebszustand = "nicht ermittelbar";
        }
        
        dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), betriebszustand));
      }
      else // keine Daten vorhanden
      {
        dataAnzeigeQuerschnittSchaltbarkeitNba.getScaledValue("DurchNbaSchaltbar").setText(durchNbaSchaltbarZustandBestimmen(anzeigeQuerschnittAktor.isImmerSchaltbar(), "nicht ermittelbar"));
      }
    }
    
    return dataAnzeigeQuerschnittSchaltbarkeitNba;
  }
  
  /**
   * Bestimmung des Zustandes des vorgelagerten Entscheiders. Hier wird die immerSchaltbar-Information vorrangig ausgewertet.
   * 
   * @param immerSchaltbar Information, ob immer schaltbar ist (<code>true</code>, wenn kein vorgelagerter Entscheider vorhanden).
   * @param zustand aktuell ermittelter Zustand.
   * @return aktuell ermittelter Zustand oder wenn immerSchaltbar <code>true</code> ist, "Ja".
   */
  public String durchNbaSchaltbarZustandBestimmen(boolean immerSchaltbar, String zustand)
  {
    if (immerSchaltbar)
    {
      return "Ja";
    }
    
    return zustand;
  }

  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal, wenn die Helligkeit geschaltet werden soll.
   * 
   * @param resultData AttributGruppe atg.anzeigeQuerschnittEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeQuerschnittEigenschaftSollHelligkeit(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;

    Data data = resultData.getData();
    
    int helligkeit = -3;
    
    if (data.getItem("Eigenschaft").getScaledValue("Helligkeit").isState()) // lokale Steuerung
    {
      if (data.getItem("Eigenschaft").getScaledValue("Helligkeit").getValueText().equals("nicht ermittelbar"))
      {
        helligkeit = -1; // lokale Helligkeitssteuerung
      }
    }
    else // Steuerung durch Zentrale
    {
      helligkeit = data.getItem("Eigenschaft").getScaledValue("Helligkeit").intValue();
    }
    
    if (helligkeit >= -1)
    {
      SystemObject objekt = resultData.getObject();
      
      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgHelligkeit"));
  
      if (helligkeit != -1)
      {
        dataErgebnis.getScaledValue("HelligkeitsWert").set(helligkeit);
        dataErgebnis.getScaledValue("Status").setText("Einstellung durch Zentrale");
      }
      else
      {
        dataErgebnis.getScaledValue("HelligkeitsWert").set(0);
        dataErgebnis.getScaledValue("Status").setText("lokale Helligkeitssteuerung mit spontaner nderungsmeldung");
      }
   
      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt().get(objekt.getPid());
      
      SystemObject de = anzeigeQuerschnittAktor.getDe();
  
      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgHelligkeit");
      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect("asp.tlsVorgabe");
  
      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), resultData.getDataTime(), dataErgebnis);
    }
    
    return ergebnis;
  }
  
  /**
   * Abarbeitung der Stromausfall-Information.
   * 
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de De die die Stromausfall-Information liefert.
   * @param stromausfall <code>true</code>, wenn Stromausfall besteht, sonst <code>false</code>.
   * @return Attributgruppe atg.anzeigeQuerschnittEigenschaftIst.
   */
  public ResultData anzeigeQuerschnittStromausfall(SystemObject anzeigeQuerschnitt, SystemObject de, Boolean stromausfall)
  {
    ResultData ergebnis = null;

    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnitt.getPid());
    
    if (stromausfall)
    {
      dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnitt.getPid(), de.getPid(), STATUS_STROMAUSFALL));
    }
    else
    {
      dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnitt.getPid(), de.getPid(), STATUS_OK));
    }
    
    AttributeGroup atg = anzeigeQuerschnitt.getDataModel().getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst");
    Aspect asp = anzeigeQuerschnitt.getDataModel().getAspect("asp.aqZustand");
    
    DataDescription dataDescription = new DataDescription(atg, asp);

    ergebnis = new ResultData(anzeigeQuerschnitt, dataDescription, 0L, dataAnzeigeQuerschnittEigenschaftIst);
    
    return ergebnis;
  }
  
  /**
   * Abarbeitung der Trkontakt-Information.
   * 
   * @param anzeigeQuerschnitt betroffener AnzeigeQuerschnitt.
   * @param de De die die Trkontakt-Information liefert.
   * @param tuerkontakt <code>true</code>, wenn Trkontakt besteht, sonst <code>false</code>.
   * @return Attributgruppe atg.anzeigeQuerschnittEigenschaftIst.
   */
  public ResultData anzeigeQuerschnittTuerkontakt(SystemObject anzeigeQuerschnitt, SystemObject de, Boolean tuerkontakt)
  {
    ResultData ergebnis = null;

    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnitt.getPid());
    
    if (tuerkontakt)
    {
      dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnitt.getPid(), de.getPid(), STATUS_TRKONTAKT));
    }
    else
    {
      dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnitt.getPid(), de.getPid(), STATUS_OK));
    }
    
    AttributeGroup atg = anzeigeQuerschnitt.getDataModel().getAttributeGroup("atg.anzeigeQuerschnittEigenschaftIst");
    Aspect asp = anzeigeQuerschnitt.getDataModel().getAspect("asp.aqZustand");
    
    DataDescription dataDescription = new DataDescription(atg, asp);

    ergebnis = new ResultData(anzeigeQuerschnitt, dataDescription, 0L, dataAnzeigeQuerschnittEigenschaftIst);
    
    return ergebnis;
  }
  
  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal, wenn die Betriebsart geschaltet werden soll.
   * 
   * @param resultData AttributGruppe atg.anzeigeQuerschnittEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeQuerschnittEigenschaftSollBetriebsart(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;

    Data data = resultData.getData();
    
    boolean beeinflussung = false;
    Betriebsart betriebsart = Betriebsart.Normalbetrieb;
    
    if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").isState())
    {
      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQNormalbetrieb"))
      {
        betriebsart = Betriebsart.Normalbetrieb;

        beeinflussung = true;
      }

      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQBlindbetrieb"))
      {
        betriebsart = Betriebsart.Blindbetrieb;

        beeinflussung = true;
      }

      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQAutarkbetrieb"))
      {
        betriebsart = Betriebsart.Autarkbetrieb;

        beeinflussung = true;
      }
    }
    
    if (beeinflussung)
    {
      SystemObject objekt = resultData.getObject();
      
      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgBetriebsArt"));
  
      if (betriebsart.equals(Betriebsart.Blindbetrieb))
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Blindbetrieb");
      }
      else if (betriebsart.equals(Betriebsart.Autarkbetrieb))
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Autarker Betrieb");
      }
      else
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Normalbetrieb");
      }
      
      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt().get(objekt.getPid());
      
      SystemObject de = anzeigeQuerschnittAktor.getDe();
  
      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgBetriebsArt");
      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect("asp.parameterVorgabe");
  
      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), resultData.getDataTime(), dataErgebnis);
    }
    
    return ergebnis;
  }
  
  /**
   * Holen der HashMap mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   * 
   * @return HashMap mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   */
  public HashMap<String, AnzeigeQuerschnittAktorTls> getListeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt()
  {
    return m_listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt;
  }

  /**
   * Holen der HashMap mit AnzeigeQuerschnittAktorTls mit Key DEs.
   * 
   * @return HashMap mit AnzeigeQuerschnittAktorTls mit Key DEs.
   */
  public HashMap<String, AnzeigeQuerschnittAktorTls> getListeAnzeigeQuerschnittAktorTlsDes()
  {
    return m_listeAnzeigeQuerschnittAktorTlsDes;
  }

  /**
   * Holen der HashMap mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   * 
   * @return HashMap mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   */
  public HashMap<String, Data> getListeAnzeigeQuerschnittEigenschaftIst()
  {
    return m_listeAnzeigeQuerschnittEigenschaftIst;
  }

  /**
   * Holen der HashMap mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   * 
   * @return HashMap mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   */
  public HashMap<String, Data> getListeAnzeigeQuerschnittSchaltbarkeitNba()
  {
    return m_listeAnzeigeQuerschnittSchaltbarkeitNba;
  }

  /**
   * Holen der HashMap fr Status Methoden fr alle AnzeigenQuerschnitte.
   * 
   * @return HashMap fr Status Methoden fr alle AnzeigenQuerschnitte.
   */
  public HashMap<String, HashMap<String, Integer>> getListeAnzeigeQuerschnittStatus()
  {
    return m_listeAnzeigeQuerschnittStatus;
  }

  /**
   * Bestimmen des aktuellen AnzeigeQuerschnitt-Status mit der Mglichkeit gleichzeitiger bergabe des Status einer Abarbeitungs-Methode.
   * 
   * @param anzeigeQuerschnittPid Pid des AnzeigeQuerschnitt.
   * @param methode Abarbeitungsmethode oder <code>null</code>, wenn nur Bestimmung des aktuellen Status.
   * @param status aktuelle Status der Abarbeitungsmethode, wenn methode nicht <code>null</code>. 
   * @return aktueller AnzeigeQuerschnitt-Status, bestimmt aus allen fr den AnzeigeQuerschnitt vorhandenen Methoden.
   */
  @SuppressWarnings("unchecked")
  protected String statusBestimmen(String anzeigeQuerschnittPid, String methode, int status)
  {
    HashMap<String, Integer> statusMethoden = (HashMap<String, Integer>)getListeAnzeigeQuerschnittStatus().get(anzeigeQuerschnittPid);
    
    if (statusMethoden == null)
    {
      statusMethoden = new HashMap<String, Integer>();
      
      getListeAnzeigeQuerschnittStatus().put(anzeigeQuerschnittPid, statusMethoden);
    }

    if (methode != null)
    {
      boolean bernehmen = true;
      
      if (bernehmen)
      {
        statusMethoden.put(methode, status); 
      }
    }
    
    Set<String> set = statusMethoden.keySet();
    Iterator iterator = set.iterator();
      
    int wert = 0;
    int wertMax = STATUS_OK;
    
    while (iterator.hasNext())
    {
      String meth = (String)iterator.next();
      
      wert = statusMethoden.get(meth);
      
      wertMax = (wert > wertMax) ? wert : wertMax;
    }
    
    return statusAusWertBestimmen(wertMax);
  }
  
  /**
   * Umsetzung des kodierten Status in den entsprechenden Text.
   * 
   * @param status kodierter Status.
   * @return dem Status entsprechender Text.
   */
  protected String statusAusWertBestimmen(int status)
  {
    String ergebnis = "OK";
    
    switch (status)
    {
      case STATUS_GESTOERT:
      {
        ergebnis = "Gestrt";

        break;
      }
      
      case STATUS_KOMMUNIKATIONSAUSFALL:
      {
        ergebnis = "Kommunikationsausfall";

        break;
      }
      
      case STATUS_STROMAUSFALL:
      {
        ergebnis = "Stromausfall";

        break;
      }
      
      case STATUS_TEILSTOERUNG:
      {
        ergebnis = "Teilstrung";

        break;
      }
      
      case STATUS_TRKONTAKT:
      {
        ergebnis = "Trkontakt";

        break;
      }
      
      case STATUS_UNDEFINIERT:
      {
        ergebnis = "Undefiniert";

        break;
      }
      
      default:
      {
        break;
      }
    }
    
    return ergebnis;
  }
}
