/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;


import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.agzs.tls.AnzeigeQuerschnittTypUmsetzungTlsDefault.Betriebsart;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der AnzeigeQuerschnitt-Informationen fr NW LEH.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2014/02/13 16:28:12 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeQuerschnittTypUmsetzungTlsNwLeh1 extends AnzeigeQuerschnittTypUmsetzungTlsDefault
{  
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeQuerschnittAktorTlsDes Liste mit AnzeigeQuerschnittAktorTls mit Key DEs.
   * @param listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt Liste mit AnzeigeQuerschnittAktorTls mit Key AnzeigeQuerschnitt.
   * @param listeAnzeigeQuerschnittEigenschaftIst Liste mit Attributgruppe atg.anzeigeQuerschnittEigenschaftIst fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittSchaltbarkeitNba Liste mit Attributgruppe atg.anzeigeQuerschnittSchaltbarkeitNba fr alle AnzeigenQuerschnitte aktuell.
   * @param listeAnzeigeQuerschnittStatus Liste mit Status Methoden fr alle AnzeigenQuerschnitte
   */
  public AnzeigeQuerschnittTypUmsetzungTlsNwLeh1(HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsDes, HashMap<String, AnzeigeQuerschnittAktorTls> listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, HashMap<String, Data> listeAnzeigeQuerschnittEigenschaftIst, HashMap<String, Data> listeAnzeigeQuerschnittSchaltbarkeitNba, HashMap<String, HashMap<String, Integer>> listeAnzeigeQuerschnittStatus)
  {
    super(listeAnzeigeQuerschnittAktorTlsDes, listeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt, listeAnzeigeQuerschnittEigenschaftIst, listeAnzeigeQuerschnittSchaltbarkeitNba, listeAnzeigeQuerschnittStatus);

    m_debug.finer("AnzeigeQuerschnittTypUmsetzungTlsNwLeh1");
  }
  
  /**
   * Abarbeitung der Betriebsart mit den Erweiterungen der LEH Kln-Lvenich.
   * Betriebsart Tunnelbetrieb wird mit Wert 9 gemeldet.
   * 
   * @param result Attributgruppe atg.tlsWzgBetriebsart.
   */
  public Data tlsWzgBetriebsArt(ResultData result)
  {
    SystemObject object = result.getObject();
    
    AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsDes().get(object.getPid());

    if (anzeigeQuerschnittAktor == null)
      return null;
    
    String anzeigeQuerschnittPid = anzeigeQuerschnittAktor.getAnzeigeQuerschnitt().getPid();
    
    Data dataAnzeigeQuerschnittEigenschaftIst = (Data)getListeAnzeigeQuerschnittEigenschaftIst().get(anzeigeQuerschnittPid);
    
    m_debug.finer("tlsWzgBetriebsart DE " + object.getPid() + "\nAnzeigeQuerschnitt " + anzeigeQuerschnittPid + "\nresult " + result + "\nData " + dataAnzeigeQuerschnittEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        String betriebszustand = "Undefiniert";
        
        if (data.getScaledValue("BetriebsArt").isState())
        {
          String betriebsart = data.getScaledValue("BetriebsArt").getValueText();
          
          if (betriebsart.equals("Normalbetrieb"))
          {
            betriebszustand = "AQNormalbetrieb";
          }
          
          if (betriebsart.equals("Blindbetrieb"))
          {
            betriebszustand = "AQBlindbetrieb";
          }
          
          if (betriebsart.equals("Handbetrieb"))
          {
            betriebszustand = "AQVorortbetrieb";
          }
          
          if (betriebsart.equals("Autarker Betrieb"))
          {
            betriebszustand = "AQAutarkbetrieb";
          }
          
          if (betriebsart.equals("Testbetrieb"))
          {
            betriebszustand = "AQTestbetrieb";
          }
          
          if (betriebsart.equals("Notbetrieb"))
          {
            betriebszustand = "AQNotbetrieb";
          }
        }
        else
        {
          int betriebsart = data.getUnscaledValue("BetriebsArt").intValue();
          
          if (betriebsart == 9) // Tunnelbetrieb
          {
            betriebszustand = "AQTunnelbetrieb";
          }
        }
        
        dataAnzeigeQuerschnittEigenschaftIst.getItem("Eigenschaft").getScaledValue("Betriebszustand").setText(betriebszustand);
        
        dataAnzeigeQuerschnittEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_OK));
      }
      else // keine Daten vorhanden
      {
        AnzeigenGlobalZwischenSchichtTls.atgAnzeigeQuerschnittEigenschaftIstInitialisieren(dataAnzeigeQuerschnittEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigeQuerschnittPid, "tlsWzgBetriebsArt", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeQuerschnittEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal, wenn die Betriebsart geschaltet werden soll.
   * 
   * @param resultData AttributGruppe atg.anzeigeQuerschnittEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeQuerschnittEigenschaftSollBetriebsart(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;

    Data data = resultData.getData();
    
    boolean beeinflussung = false;
    Betriebsart betriebsart = Betriebsart.Normalbetrieb;
    
    if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").isState())
    {
      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQNormalbetrieb"))
      {
        betriebsart = Betriebsart.Normalbetrieb;

        beeinflussung = true;
      }

      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQBlindbetrieb"))
      {
        betriebsart = Betriebsart.Blindbetrieb;

        beeinflussung = true;
      }

      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQAutarkbetrieb"))
      {
        betriebsart = Betriebsart.Autarkbetrieb;

        beeinflussung = true;
      }

      if (data.getItem("Eigenschaft").getScaledValue("Betriebszustand").getValueText().equals("AQTunnelbetrieb"))
      {
        betriebsart = Betriebsart.Tunnelbetrieb;

        beeinflussung = true;
      }
    }
    
    if (beeinflussung)
    {
      SystemObject objekt = resultData.getObject();
      
      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgBetriebsArt"));
  
      if (betriebsart.equals(Betriebsart.Blindbetrieb))
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Blindbetrieb");
      }
      else if (betriebsart.equals(Betriebsart.Autarkbetrieb))
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Autarker Betrieb");
      }
      else if (betriebsart.equals(Betriebsart.Tunnelbetrieb))
      {
        dataErgebnis.getUnscaledValue("BetriebsArt").set(9);
      }
      else
      {
        dataErgebnis.getScaledValue("BetriebsArt").setText("Normalbetrieb");
      }
      
      AnzeigeQuerschnittAktorTls anzeigeQuerschnittAktor = (AnzeigeQuerschnittAktorTls)getListeAnzeigeQuerschnittAktorTlsAnzeigeQuerschnitt().get(objekt.getPid());
      
      SystemObject de = anzeigeQuerschnittAktor.getDe();
  
      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.tlsWzgBetriebsArt");
      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect("asp.parameterVorgabe");
  
      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), resultData.getDataTime(), dataErgebnis);
    }
    
    return ergebnis;
  }
}
