/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;


import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr NRW.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.3 $ / $Date: 2012/05/23 15:47:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsNrw1 extends AnzeigeTypUmsetzungTlsDefault
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsNrw1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsNrw1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }

  /**
   * Abarbeitung des TLS-Wechseltextes entsprechend den dWista-Vorgaben NRW.
   * 
   * D.h. in WVZTextZeichen die ersten 6 Zeichen Zusatzinformationen und dann folgen die sichtbaren Zeichen
   * 
   * @param result Attributgruppe atg.tlsWzgWechselText
   */
  public Data tlsWzgWechselText(ResultData result)
  {
    SystemObject object = result.getObject();
    
    DataModel dataModel = object.getDataModel();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsDes().get(object.getPid());

    if (anzeigeAktor == null)
      return null;
    
    String anzeigePid = anzeigeAktor.getAnzeige().getPid();
    
    Data dataAnzeigeEigenschaftIst = (Data)getListeAnzeigeEigenschaftIst().get(anzeigePid);
    
    m_debug.finer("tlsWzgWechselText DE " + object.getPid() + "\nAnzeige " + anzeigePid + "\nresult " + result + "\nData " + dataAnzeigeEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeEigenschaftIstInitialisieren(dataAnzeigeEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigePid, "tlsWzgWechselText", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();

        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();

//  Ergebnis lschen    

        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);

//  Grundfont bestimmen    
    
        String pidGrundFontUmsetzungTls = anzeigeTypUmsetzungTls.getPidGrundFont();
    
        String zeichenfolge = data.getTextValue("WVZTextZeichen").getValueText();
        int laengeInput = zeichenfolge.length();
        int laengeText = laengeInput-6;
  
        if (laengeInput == 0) // Anzeige ausgeschaltet
        {
          String textAusrichtung = anzeigeTypUmsetzungTls.getGrundTextAusrichtung();
          String pidGrundFont = anzeigeTypUmsetzungTls.getPidGrundFont();
          
          Data dataDynamischerWechseltext = dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext");
          
          dataDynamischerWechseltext.getReferenceValue("Font").setSystemObjectPid(pidGrundFont, dataModel);
          dataDynamischerWechseltext.getScaledValue("TextAusrichtung").setText(textAusrichtung);
          dataDynamischerWechseltext.getReferenceArray("Textdefinition").setLength(0);
          
          dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
        }
        else
        {  

//      Font aus Rckmeldung bestimmen    
      
          String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, zeichenfolge.codePointAt(0), pidGrundFontUmsetzungTls);
      
          FontUmsetzungTls fontUmsetzungTls = null;
  
          if (pidFontUmsetzungTls != null)
          {
            HashMap mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
          
            if (mapFont != null)
              fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontUmsetzungTls);
          }
  
          HashMap mapZeichen = null;
        
          if (fontUmsetzungTls != null)
            mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsCode();
  
          if ((mapZeichen == null) || (mapZeichen.size() == 0))
            mapZeichen = anzeigeTypUmsetzungTls.getListeZeichenUmsetzungTlsCode();
  
//        Pid Font ber Umsetzung bestimmen
      
          String pidFont = null;
        
          if (fontUmsetzungTls != null)
          {
            pidFont = fontUmsetzungTls.getPidFont();
          }
          
          if (laengeText < 0)
          {
            laengeText = 0;
    
            m_debug.warning(object.getPid() + "Wechseltext zu kurz (6 + Text) '" + zeichenfolge + "'");
            
            return null;
          }
          
          if (pidFont != null)
          {
            dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setText(pidFont);
          }

//      Textausrichtung aus Rckmeldung bestimmen
        
          String textAusrichtung = textAusrichtungAusTlsBestimmen(anzeigeTypUmsetzungTls, zeichenfolge.codePointAt(1));
          
          dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").setText(textAusrichtung);
        
//      Text        
        
          dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(laengeText);
  
          for (int i = 6; i < zeichenfolge.length(); i++)
          {
            m_debug.finest("Zeichen " + i + " " + zeichenfolge.codePointAt(i));
    
            int code = zeichenfolge.codePointAt(i);
    
            ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(code);
            
            if (zeichenUmsetzungTls != null)
              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
            else
            {
              m_debug.error("Code " + code + " an Position " + i + " nicht umsetzbar an Anzeige " + anzeigePid + " Leerzeichen wird verwendet");

              zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(32); // Leerzeichen als Default

              if (zeichenUmsetzungTls != null)
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
              else
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setSystemObject(null);
            }
          }
        }

        tlsWzgWechselTextBlinken(data, dataAnzeigeEigenschaftIst);

        dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgWechselText", STATUS_OK));
      }
      else // keine Daten vorhanden
      {
        tlsWzgWechselTextBlinken(null, dataAnzeigeEigenschaftIst);

        dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgWechselText", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal.
   * Speziell fr NRW.
   * 
   * @param result AttributGruppe atg.anzeigeEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeEigenschaftSoll(ResultData result, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;
    
    SystemObject objekt = result.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsAnzeige().get(objekt.getPid());

    if (anzeigeAktor == null)
      return null;
    
    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
    
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();
    
        long zeitpunkt = data.getTimeValue("SollZeitpunkt").getMillis();

        ConfigurationObject wechseltext = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("Wechseltext").getSystemObject();

        Data dataWechseltext = null;
        
        if (wechseltext != null) // konfigurierter Wechseltext
        {
          AttributeGroup atgWechseltext = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wechseltext");
          dataWechseltext = wechseltext.getConfigurationData(atgWechseltext);
        }
        else // dynamischer Wechseltext
        {
          dataWechseltext = data.getItem("Eigenschaft").getItem("DynamischerWechseltext");
        }
        
        String pidFont = dataWechseltext.getReferenceValue("Font").getValueText();
        int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidFont);

//        System.out.println("codeFont " + codeFont);
        
        String pidTextAusrichtung = dataWechseltext.getScaledValue("TextAusrichtung").getValueText();
        int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidTextAusrichtung);

//        System.out.println("codeTextAusrichtung " + codeTextAusrichtung);

        HashMap mapZeichen = null;
        
        HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
        
        if ((pidFont != null) && (mapFont != null))
        {
          TreeSet<String> treeset = new TreeSet<String>();
          treeset.addAll(mapFont.keySet());
        
          Iterator iterator = treeset.iterator();
        
          FontUmsetzungTls fontUmsetzungTls;
          
          while (iterator.hasNext())
          {
            fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
            
            if (fontUmsetzungTls.getPidFont().equals(pidFont))
            {
              mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsZeichen();
              
              break;
            }
          }
        }

        if (isKannFreiText()) // reine Textansteuerung
        {
          // DEs bestimmen
          
          int anzahlDes = anzeigeAktor.getDes().size();
          
          if (anzahlDes == 1)
          {
            Set set = anzeigeAktor.getDes().keySet();
            
            Iterator it = set.iterator();
            
            String key = (String)it.next();
            
            SystemObject de = anzeigeAktor.getDes().get(key);
            
            int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

            for (int j = 0; j < anzahlGlobalUmsetzung; j++)
            {
              GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

              for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
              {
                GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                
                String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();

                if (asp.length() > 0)
                {
                  if ((atg.equals("atg.tlsWzgWechselText")) ||
                      (atg.equals("atg.tlsWzgStellZustand")))
                  {
//                    System.out.println("ATG " + atg + " gefunden, nun fllen (und versenden)");
                   
                    AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                    Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                    
                    Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                    if (atg.equals("atg.tlsWzgWechselText"))
                    {
                      StringBuffer befehl = new StringBuffer();
                      
                      befehl.append(String.format("%c", codeFont));
                      befehl.append(String.format("%c", codeTextAusrichtung));
                      befehl.append(String.format("%c", 0));
                      befehl.append(String.format("%c", 0));
                      befehl.append(String.format("%c", 0));
                      befehl.append(String.format("%c", 0));

                      ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                      for (int i = 0; i < text.length; i++)
                      {
                        String pid = text[i].getSystemObject().getPid();
                        
                        if (mapZeichen == null)
                        {
                          continue;
                        }
                        
                        ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                        
                        if (zeichenUmsetzungTls != null)
                        {
                          befehl.append(String.format("%c", zeichenUmsetzungTls.getZeichencode()));
                        }
                        else
                        {
                          befehl.append(String.format("%c", 32)); // Leerzeichen als default
                        }
                      }

                      dataErgebnis.getTextValue("WVZTextZeichen").setText(befehl.toString());

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                      
                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }

                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    break;
                  }
                }
              }
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }
  
  /**
   * Bestimmung der Text-Ausrichtung, kann sie nicht bestimmt werden, wird die Text-Ausrichtung 'Undefiniert' zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @return Text-Ausrichtung.
   */
  private String textAusrichtungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code)
  {
    String ergebnis = "Undefiniert";

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if (mapTextAusrichtung != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (code == textAusrichtungUmsetzungTls.getCodeTextAusrichtung())
        {
          ergebnis = textAusrichtungUmsetzungTls.getTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code der Text-Ausrichtung, kann sie nicht bestimmt werden, wird Code 0 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param pidAusrichtung Pid der TextAusrichtung
   * @return Code der TextAusrichtung
   */
  private int textAusrichtungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String pidAusrichtung)
  {
    int ergebnis = 133; // Linksbndig als Default

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if ((pidAusrichtung != null) && (mapTextAusrichtung != null))
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (textAusrichtungUmsetzungTls.getTextAusrichtung().equals(pidAusrichtung))
        {
          ergebnis = textAusrichtungUmsetzungTls.getCodeTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung der Pid der FontUmsetzungTls, kann er nicht bestimmt werden, wird die Pid des GrundFontUmsetzungTls zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @param pidGrundFontUmsetzungTls Pid des GrundFonts der FontUmsetzungTls.
   * @return Pid der FontUmsetzungTls.
   */
  private String fontUmsetzungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code, String pidGrundFontUmsetzungTls)
  {
    String ergebnis = pidGrundFontUmsetzungTls;
    
    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if (mapFont != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapFont.keySet());
    
      Iterator iterator = treeset.iterator();
    
      FontUmsetzungTls fontUmsetzungTls;
      
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (code == fontUmsetzungTls.getCodeFont())
        {
          ergebnis = fontUmsetzungTls.getPid();
          
          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code der FontUmsetzungTls, kann er nicht bestimmt werden, wird der Code 0 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param pidFont Pid des Font
   * @return Code des Font
   */
  private int fontUmsetzungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String pidFont)
  {
    int ergebnis = 128; // Mittelschrift als Default

    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if ((pidFont != null) && (mapFont != null))
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapFont.keySet());
    
      Iterator iterator = treeset.iterator();
    
      FontUmsetzungTls fontUmsetzungTls;
      
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (fontUmsetzungTls.getPidFont().equals(pidFont))
        {
          ergebnis = fontUmsetzungTls.getCodeFont();
          
          break;
        }
      }
    }
    
    return ergebnis;
  }
}
