/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr das WTA-C-Zeichen in NW.
 * Dieses C-Zeichen kann mit Codes und mit Wechseltext angesteuert werden (Aussenanlage TLS 2002 und hher).
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2014/02/13 16:28:12 $ / ($Author: Drapp $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsNw1CZeichen extends AnzeigeTypUmsetzungTlsBw1CZeichen
{
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsNw1CZeichen(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsNw1CZeichen(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }
}
