/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.dirigentAktorTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2012/03/09 10:55:21 $ / ($Author: Drapp $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Thomas Drapp
 *
 */
public class DirigentAktorTls
{
  /**
   * Konfigurationsobjekt vom Typ typ.dirigentAktorTls.
   */
  SystemObject m_dirigentAktorTls;
  /**
   * Konfigurationsobjekt vom Typ typ.dirigent das der Aktor umsetzt.
   */
  SystemObject m_dirigent;
  /**
   * Instanz der Klasse mit den Umsetzung-Informationen.
   */
  DirigentUmsetzungTls m_dirigentUmsetzung;
  /**
   * DE die den Aktor realisieren.
   */
  SystemObject m_de;
  /**
   * Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  SystemObject m_eak;
  
  /**
   * Konstruktor mit bergabe des Konfigurationsobjektes.
   * 
   * @param dirigentAktorTls Konfigurationsobjekt ber das der Aktor versorgt ist.
   */
  public DirigentAktorTls(SystemObject dirigentAktorTls)
  {
    m_dirigentAktorTls = dirigentAktorTls;
  }
  
  /**
   * Holen der Pid des DirigentAktorTls.
   * 
   * @return Pid des DirigentAktorTls.
   */
  public String getPid()
  {
    if (m_dirigentAktorTls != null)
      return m_dirigentAktorTls.getPid();
    else
      return null;
  }

  /**
   * Holen des Dirigenten den der Aktor umsetzt.
   * 
   * @return Dirigent den der Aktor umsetzt.
   */
  public SystemObject getDirigent()
  {
    return m_dirigent;
  }
  
  /**
   * Setzen des Dirigenten den der Aktor umsetzt.
   * 
   * @param dirigent Dirigent den der Aktor umsetzt.
   */
  public void setDirigent(SystemObject dirigent)
  {
    m_dirigent = dirigent;
  }
  
  /**
   * Holen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @return Instanz der Klasse mit den Umsetzung-Iinformationen.
   */
  public DirigentUmsetzungTls getDirigentUmsetzung()
  {
    return m_dirigentUmsetzung;
  }
  
  /**
   * Setzen der Instanz der Klasse mit den Umsetzung-Informationen.
   * 
   * @param dirigentUmsetzung Instanz der Klasse mit den Umsetzung-Informationen.
   */
  public void setDirigentUmsetzung(DirigentUmsetzungTls dirigentUmsetzung)
  {
    m_dirigentUmsetzung = dirigentUmsetzung;
  }
  
  /**
   * Holen des Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   * 
   * @return Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   */
  public SystemObject getDe()
  {
    return m_de;
  }
  
  /**
   * Setzen des Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   * 
   * @param de Konfigurationsobjekt vom Typ typ.de die den Aktor realisiert.
   */
  public void setDe(SystemObject de)
  {
    m_de = de;
  }

  /**
   * Holen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   * 
   * @return Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public SystemObject getEak()
  {
    return m_eak;
  }

  /**
   * Setzen des Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   * 
   * @param eak Konfigurationsobjekt vom Typ typ.eak an den die DE gekoppelt ist.
   */
  public void setEak(SystemObject eak)
  {
    m_eak = eak;
  }

  /* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    String string = new String();

    string += "DirigentAktorTls";
    string += " pid " + getPid();

    if (m_dirigent != null)
    	string += ", Dirigent " + m_dirigent.getPid();

    if (m_dirigentUmsetzung != null)
    	string += ", DirigentUmsetzung " + m_dirigentUmsetzung.getPid();
    
    string += ", DE: " + m_de.getPid();
    
    return string;
  }
}
