/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.fontUmsetzungTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class FontUmsetzungTls
{
  /**
   * Datenmodell.
   */
  DataModel m_dataModel;
  /**
   * Pid des Konfigurationsobjektes vom Typ typ.fontUmsetzungTls.
   */
  String m_pid;
  /**
   * Konfigurationsobjekt vom Typ typ.fontUmsetzung.
   */
  SystemObject m_fontUmsetzung;
  /**
   * HashMap mit ZeichenUmsetzungTls mit Key Pid des Zeichen.
   */
  HashMap<String, ZeichenUmsetzungTls> m_listeZeichenUmsetzungTlsZeichen = new HashMap<String, ZeichenUmsetzungTls>();
  /**
   * HashMap mit ZeichenUmsetzungTls mit Key Code des Zeichen.
   */
  HashMap<Integer, ZeichenUmsetzungTls> m_listeZeichenUmsetzungTlsCode = new HashMap<Integer, ZeichenUmsetzungTls>();
  /**
   * Vector mit Instanzen der Klasse GlobalUmsetzung mit allen Umsetzungen.
   */
  Vector<GlobalUmsetzung> m_listeFontUmsetzung = new Vector<GlobalUmsetzung>();
  /**
   * Code des Font auf TLS-Seite fr den Font
   */
  int m_codeFont;
  /**
   * Pid des Fonts auf AnzeigenGlobal-Seite
   */
  String m_pidFont;

  /**
   * Konstruktor mit bergabe des Daten-Modell, der Pid der FontUmsetzung und dem Konfigurationsobjekt vom Typ typ.fontUmsetzung. 
   * 
   * @param dataModel Daten-Modell.
   * @param pid Pid der FontUmsetzung.
   * @param fontUmsetzung Konfigurationsobjekt vom Typ typ.fontUmsetzung.
   */
  public FontUmsetzungTls(DataModel dataModel, String pid, SystemObject fontUmsetzung)
  {
    int zeichencode;

    m_dataModel = dataModel;
    m_pid = pid;
    m_fontUmsetzung = fontUmsetzung;
    
    AttributeGroup atgFontUmsetzung       = m_dataModel.getAttributeGroup("atg.fontUmsetzung");
    AttributeGroup atgFontUmsetzungTls    = m_dataModel.getAttributeGroup("atg.fontUmsetzungTls");
    AttributeGroup atgZeichenUmsetzung    = m_dataModel.getAttributeGroup("atg.zeichenUmsetzung");
    AttributeGroup atgZeichenUmsetzungTls = m_dataModel.getAttributeGroup("atg.zeichenUmsetzungTls");

    Data datenAtgFontUmsetzung    = m_fontUmsetzung.getConfigurationData(atgFontUmsetzung);
    Data datenAtgFontUmsetzungTls = m_fontUmsetzung.getConfigurationData(atgFontUmsetzungTls);
    
    if (datenAtgFontUmsetzungTls != null)
    {
      int anzahlUmsetzungen = datenAtgFontUmsetzungTls.getArray("UmsetzungFont").getLength();
      
      for (int i = 0; i < anzahlUmsetzungen; i++)
      {
        GlobalUmsetzung globalUmsetzung = new GlobalUmsetzung(datenAtgFontUmsetzungTls.getArray("UmsetzungFont").getItem(i));

        getListeFontUmsetzung().add(globalUmsetzung);
      }
    }

    zeichencode = 0;
    
    if (getListeFontUmsetzung().size() >= 1)
    {
      long[] zeichen = getListeFontUmsetzung().get(0).getTemplate();
      
      if (zeichen.length >= 1)
      {
        zeichencode = (int)zeichen[0];
      }
    }
    
//    System.out.println("Font " + datenAtgFontUmsetzung.getTextValue("FontReferenz") + " zeichen " + zeichencode);
    
    setPidFont(datenAtgFontUmsetzung.getTextValue("FontReferenz").getValueText());
    setCodeFont(zeichencode);
    
//    if (getPidFont().equals("font.dWiSta.KlnPilot.Schmalschrift"))
//    {
//      setCodeFont(129);
//    }
//    else
//    {
//      setCodeFont(128);
//    }
    
    int anzahl = datenAtgFontUmsetzung.getReferenceArray("ZeichenUmsetzungReferenz").getLength();
    
    SystemObject umsetzung;
    Data datenAtgZeichenUmsetzung;
    Data datenAtgZeichenUmsetzungTls;
    String zeichenPid;
    
    for (int i = 0; i < anzahl; i++)
    {
      umsetzung = datenAtgFontUmsetzung.getReferenceArray("ZeichenUmsetzungReferenz").getReferenceValue(i).getSystemObject();
      
      datenAtgZeichenUmsetzung    = umsetzung.getConfigurationData(atgZeichenUmsetzung);
      datenAtgZeichenUmsetzungTls = umsetzung.getConfigurationData(atgZeichenUmsetzungTls);
      
      zeichenPid  = datenAtgZeichenUmsetzung.getReferenceValue("ZeichenReferenz").getValueText();
      zeichencode = datenAtgZeichenUmsetzungTls.getUnscaledValue("Zeichencode").intValue();
      
//      System.out.println("FONTUMSETZUNGTLS Zeichenumsetzung '" + zeichenPid + "' " + zeichencode);
      
      ZeichenUmsetzungTls zeichenUmsetzungTls = new ZeichenUmsetzungTls(zeichenPid, zeichencode);
      
      m_listeZeichenUmsetzungTlsZeichen.put(zeichenPid, zeichenUmsetzungTls);
      m_listeZeichenUmsetzungTlsCode.put(zeichencode, zeichenUmsetzungTls);
    }
  }
  
  /**
   * Holen der Pid der FontUmsetzungTls.
   * 
   * @return Pid der FontUmsetzungTls.
   */
  public String getPid()
  {
    return m_pid;
  }

  /**
   * Setzen der Pid der FontUmsetzungTls.
   * 
   * @param pid Pid der FontUmsetzungTls.
   */
  public void setPid(String pid)
  {
    m_pid = pid;
  }

  /**
   * Holen des Konfigurationsobjektes vom Typ typ.fontUmsetzung.
   * 
   * @return Konfigurationsobjekt vom Typ typ.fontUmsetzung.
   */
  public SystemObject getFontUmsetzung()
  {
    return m_fontUmsetzung;
  }

  /**
   * Setzen des Konfigurationsobjektes vom Typ typ.fontUmsetzung.
   * 
   * @param fontUmsetzung Konfigurationsobjekt vom Typ typ.fontUmsetzung.
   */
  public void setAnzeigeTypUmsetzung(SystemObject fontUmsetzung)
  {
    m_fontUmsetzung = fontUmsetzung;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlssel Code des Zeichen.
   * 
   * @return HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlssel Code des Zeichen.
   */
  public HashMap getListeZeichenUmsetzungTlsCode()
  {
    return m_listeZeichenUmsetzungTlsCode;
  }

  /**
   * Holen der HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlssel Pid des Zeichen.
   * 
   * @return HashMap mit Instanzen der Klasse ZeichenUmsetzungTls mit dem Schlssel Pid des Zeichen.
   */
  public HashMap getListeZeichenUmsetzungTlsZeichen()
  {
    return m_listeZeichenUmsetzungTlsZeichen;
  }

  /**
   * Holen der Pid des Fonts der umgesetzt wird.
   * 
   * @return Pid des Fonts der umgesetzt wird.
   */
  public String getPidFont()
  {
    return m_pidFont;
  }

  /**
   * Setzen der Pid des Fonts der umgesetzt wird.
   * 
   * @param pidFont Pid des Fonts der umgesetzt wird.
   */
  public void setPidFont(String pidFont)
  {
    m_pidFont = pidFont;
  }

  /**
   * Holen des Code des Fonts der umgesetzt wird.
   * 
   * @return Code des Fonts der umgesetzt wird.
   */
  public int getCodeFont()
  {
    return m_codeFont;
  }

  /**
   * Setzen des Code des Fonts der umgesetzt wird.
   * 
   * @param codeFont Code des Fonts der umgesetzt wird.
   */
  public void setCodeFont(int codeFont)
  {
    m_codeFont = codeFont;
  }

  /**
   * Holen des Vector mit Instanzen der Klasse FontUmsetzung.
   * 
   * @return Vector mit Instanzen der Klasse FontUmsetzung.
   */
  public Vector<GlobalUmsetzung> getListeFontUmsetzung()
  {
    return m_listeFontUmsetzung;
  }

  /**
   * Setzen des Vector mit Instanzen der Klasse FontUmsetzung.
   * 
   * @param listeFontUmsetzung Vector mit Instanzen der Klasse FontUmsetzung.
   */
  public void setListeFontUmsetzung(Vector<GlobalUmsetzung> listeFontUmsetzung)
  {
    m_listeFontUmsetzung = listeFontUmsetzung;
  }
}
