/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr die Speicherung der Informationen eines Konfigurationsobjektes vom Typ typ.textAusrichtungUmsetzungTls.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/05/23 15:47:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class TextAusrichtungUmsetzungTls
{
  /**
   * Datenmodell.
   */
  protected DataModel m_dataModel;
  /**
   * Pid des Konfigurationsobjektes vom Typ typ.textAusrichtungUmsetzungTls.
   */
  protected String m_pid;
  /**
   * Konfigurationsobjekt vom Typ typ.textAusrichtungUmsetzungTls.
   */
  protected SystemObject m_textAusrichtungUmsetzung;
  /**
   * Vector mit Instanzen der Klasse GlobalUmsetzung mit den TextAusrichtungUmsetzungen.
   */
  protected Vector<GlobalUmsetzung> m_listeTextAusrichtungUmsetzung = new Vector<GlobalUmsetzung>();
  /**
   * Code der Text-Ausrichtung auf TLS-Seite fr die Text-Ausrichtung.
   */
  protected int m_codeTextAusrichtung;
  /**
   * Pid der Text-Ausrichtung auf AnzeigenGlobal-Seite.
   */
  protected String m_textAusrichtung;
 
  /**
   * Konstruktor mit bergabe des Daten-Modell, der Pid der AnzeigeTypUmsetzung und dem Konfigurationsobjekt vom Typ typ.anzeigeTypUmsetzung. 
   * 
   * @param dataModel Daten-Modell.
   * @param pid Pid der TextAusrichtungUmsetzung.
   * @param textAusrichtungUmsetzung Konfigurationsobjekt vom Typ typ.textAusrichtungUmsetzung.
   */
  public TextAusrichtungUmsetzungTls(DataModel dataModel, String pid, SystemObject textAusrichtungUmsetzung)
  {
    int ausrichtcode;

    m_dataModel = dataModel;
    m_pid = pid;
    m_textAusrichtungUmsetzung = textAusrichtungUmsetzung;
    
    AttributeGroup atgTextAusrichtungUmsetzung    = m_dataModel.getAttributeGroup("atg.textAusrichtungUmsetzung");
    AttributeGroup atgTextAusrichtungUmsetzungTls = m_dataModel.getAttributeGroup("atg.textAusrichtungUmsetzungTls");

    Data datenAtgTextAusrichtungUmsetzung    = m_textAusrichtungUmsetzung.getConfigurationData(atgTextAusrichtungUmsetzung);
    Data datenAtgTextAusrichtungUmsetzungTls = m_textAusrichtungUmsetzung.getConfigurationData(atgTextAusrichtungUmsetzungTls);
    
    if (datenAtgTextAusrichtungUmsetzungTls != null)
    {
      int anzahlUmsetzungen = datenAtgTextAusrichtungUmsetzungTls.getArray("UmsetzungTextAusrichtung").getLength();
      
      for (int i = 0; i < anzahlUmsetzungen; i++)
      {
        GlobalUmsetzung globalUmsetzung = new GlobalUmsetzung(datenAtgTextAusrichtungUmsetzungTls.getArray("UmsetzungTextAusrichtung").getItem(i));

        getListeTextAusrichtungUmsetzung().add(globalUmsetzung);
      }
    }

    ausrichtcode = 0;
    
    if (getListeTextAusrichtungUmsetzung().size() >= 1)
    {
      long[] zeichen = getListeTextAusrichtungUmsetzung().get(0).getTemplate();
      
      if (zeichen.length >= 1)
      {
        ausrichtcode = (int)zeichen[0];
      }
    }
    
    setTextAusrichtung(datenAtgTextAusrichtungUmsetzung.getTextValue("TextAusrichtung").getValueText());
    setCodeTextAusrichtung(ausrichtcode);
  }
  
  /**
   * Holen der Pid der TextAusrichtungUmsetzungTls.
   * 
   * @return Pid der TextAusrichtungUmsetzungTls.
   */
  public String getPid()
  {
    return m_pid;
  }

  /**
   * Setzen der Pid der TextAusrichtungUmsetzungTls.
   * 
   * @param pid Pid der TextAusrichtungUmsetzungTls.
   */
  public void setPid(String pid)
  {
    m_pid = pid;
  }

  /**
   * Holen des Konfigurationsobjektes vom Typ typ.textAusrichtungUmsetzung.
   * 
   * @return Konfigurationsobjekt vom Typ typ.textAusrichtungUmsetzung.
   */
  public SystemObject getTextAusrichtungUmsetzung()
  {
    return m_textAusrichtungUmsetzung;
  }

  /**
   * Setzen des Konfigurationsobjektes vom Typ typ.textAusrichtungUmsetzung.
   * 
   * @param textAusrichtungUmsetzung Konfigurationsobjekt vom Typ typ.textAusrichtungUmsetzung.
   */
  public void setTextAusrichtungUmsetzung(SystemObject textAusrichtungUmsetzung)
  {
    m_textAusrichtungUmsetzung = textAusrichtungUmsetzung;
  }

  /**
   * Holen des Code der Text-Ausrichtung auf TLS-Seite fr die Text-Ausrichtung.
   * 
   * @return Code der Text-Ausrichtung auf TLS-Seite fr die Text-Ausrichtung.
   */
  public int getCodeTextAusrichtung()
  {
    return m_codeTextAusrichtung;
  }

  /**
   * Setzen des Code der Text-Ausrichtung auf TLS-Seite fr die Text-Ausrichtung.
   * 
   * @param codeTextAusrichtung Code der Text-Ausrichtung auf TLS-Seite fr die Text-Ausrichtung.
   */
  public void setCodeTextAusrichtung(int codeTextAusrichtung)
  {
    m_codeTextAusrichtung = codeTextAusrichtung;
  }

  /**
   * Holen der Text-Ausrichtung auf AnzeigenGlobal-Seite.
   * 
   * @return Text-Ausrichtung auf AnzeigenGlobal-Seite.
   */
  public String getTextAusrichtung()
  {
    return m_textAusrichtung;
  }

  /**
   * Setzen der Text-Ausrichtung auf AnzeigenGlobal-Seite.
   * 
   * @param textAusrichtung Text-Ausrichtung auf AnzeigenGlobal-Seite.
   */
  public void setTextAusrichtung(String textAusrichtung)
  {
    m_textAusrichtung = textAusrichtung;
 }

  /**
   * Holen des Vector mit Instanzen der Klasse GlobalUmsetzung mit den TextAusrichtungUmsetzungen.
   * 
   * @return Vector mit Instanzen der Klasse GlobalUmsetzung mit den TextAusrichtungUmsetzungen.
   */
  public Vector<GlobalUmsetzung> getListeTextAusrichtungUmsetzung()
  {
    return m_listeTextAusrichtungUmsetzung;
  }

  /**
   * Setzen des Vector mit Instanzen der Klasse GlobalUmsetzung mit den TextAusrichtungUmsetzungen.
   * 
   * @param listeTextAusrichtungUmsetzung Vector mit Instanzen der Klasse GlobalUmsetzung mit den TextAusrichtungUmsetzungen.
   */
  public void setListeTextAusrichtungUmsetzung(Vector<GlobalUmsetzung> listeTextAusrichtungUmsetzung)
  {
    m_listeTextAusrichtungUmsetzung = listeTextAusrichtungUmsetzung;
  }
}
