/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls.kex;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr das Publizieren von Daten.
 * Diese Klasse dient als Superklasse und wird selbst nicht instanziiert.
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:50 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class KexInput implements ClientSenderInterface
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug _debug = Debug.getLogger();
  
  /**
   * Senden der bergebenen Daten zum DAV, wenn nicht angemeldet, kein Versuch der Anmeldung.
   * 
   * @param connection Datenverteilerverbindung.
   * @param objekt Objekt fr das die Daten publiziert werden.
   * @param atg AttributGruppe der zu publizierenden Daten.
   * @param asp Aspekt der zu publizierenden Daten.
   * @param result zu publizierenden Daten.
   * @return 1 wenn daten publiziert, sonst 0.
   */
  public int publizieren(ClientDavConnection connection, SystemObject objekt, AttributeGroup atg, Aspect asp, ResultData result)
  {
    int rueckgabe = 0;
    int zaehler = 0;
    
    while (true)
    {
      
// Daten senden
      
      try
      {
        connection.sendData(result);
 
        _debug.finest("Daten gesendet " + objekt.getPid() + " result " + result);
        
        rueckgabe = 1;
        
        break;
      } 
      catch (DataNotSubscribedException e1)
      {
        e1.printStackTrace();

        zaehler++;
        
        _debug.finest("Daten nicht zum Senden angemeldet (" + zaehler + ")");

        break;
      } 
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
        
        break;
      } 
      catch (IllegalArgumentException e1)
      {
        _debug.error("IllegalArgumentException " + exceptionMeldungBestimmen(e1) + " " + result);

        e1.printStackTrace();

        break;
      }
      catch (Exception e1)
      {
        _debug.error("Exception " + exceptionMeldungBestimmen(e1) + " " + result);

        e1.printStackTrace();

        break;
      }
    }
    
    return(rueckgabe);
  }

  /* (non-Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription, byte)
   */
  public void dataRequest(SystemObject arg0, DataDescription arg1, byte arg2)
  {
  }

  /* (non-Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.dav.daf.main.config.SystemObject, de.bsvrz.dav.daf.main.DataDescription)
   */
  public boolean isRequestSupported(SystemObject arg0, DataDescription arg1)
  {
    return false;
  }

  /**
   * Informationen einer Exception in einen String packen.<br>
   * Es werden die Exception-Klasse, die Exception-Meldung und der StackTrace verarbeitet.<br>
   * 
   * @param exception zu verarbeitende Exception.
   * @return Informationen der verarbeiteten Exception. 
   */
  private String exceptionMeldungBestimmen(Exception exception)
  {
    String message = exception.getClass().getName() + " " + exception.getMessage();
    
    StackTraceElement[] stack = exception.getStackTrace();
    
    for (int i = 0; i < stack.length; i++)
    {
      message += ("\n" + stack[i]);
    }
    
    return message;
  }
}
